# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-11-17 08:31:37 +0900 (Mon, 17 Nov 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/common'

module VFS
  class Dir
    include Enumerable
    include VFS::Common
    extend  VFS::MethodGenerator
    toplevel_methods_to_instance_methods :join, :scheme, :escape, :unescape

    raise_unsupported :delete, :unlink, :rmdir, :mkdir, :entries, :chdir, :chroot

    class << self
      extend  VFS::MethodGenerator

      def new(uri, *args)
	# fixme; split this to common to share with VFS::File
	if self == VFS::Dir
          if ! uri.kind_of?(URI)
            uri = VFS.path_to_uri(uri)
          end
	  uri.scheme ||= "file"
	  begin
	    require "vfs/scheme/#{uri.scheme}/dir"
	  rescue LoadError
	    raise Error::Unsupported,
	      "scheme '#{uri.scheme}' is not supported yet!"
	  end
	  handler = "::VFS::SchemeHandler::#{uri.scheme.upcase}::Dir"
	  obj = eval(handler).send :new, uri
	  unless obj
	    raise Error::Unsupported,
	      "scheme '#{uri.scheme}' is not supported yet!"
	  end
	  return obj
	else
	  super
	end # self == VFS::Dir
      end # new

      def pwd
        # FIXME.
        DirBuiltin.pwd
      end
      alias :getwd :pwd

    end # class << self

    def initialize(uri)
      @uri     = uri.kind_of?(URI) ? uri : URI.parse(uri)
      @pos     = 0
      @entries = nil
      @is_open = false
    end

    attr_accessor :is_open, :pos
    attr_writer :entries
    private :entries=, :is_open, :is_open=
    attr_reader :uri

    def kind_of?(c)
      if c == Dir
        return true
      end
      super
    end

    def to_file
      VFS::File.new(uri)
    end

    def path
#      uri.path
      uri.to_s
    end

    def rewind
      pos = 0
      self
    end

    def seek(p)
      pos = p
      self
    end

    def each(&block)
      rewind
      while e = read
	yield(e)
      end
    end
    alias :foreach :each

    def read
      c = entries[pos]
      @pos += 1
      return c
    end

    def open(*arg, &block)
      log("You should omit Dir.open(). This is only for backword compatibility",
	  Log::WARNING)
      is_open = true
      if block_given?
	begin
	  yield(self)
	ensure
	  close
	end
      else
	self
      end
    end

    def close
      log("You should omit Dir.close(). This is only for backword compatibility",
	  Log::WARNING)
      is_open = false
      # nil
      self # break compatibility, Hmmmmm...
    end

    def glob(pat, flag=nil)
      # FIXME!
      raise Error::Unsupported
    end

    def [](pat)
      glob(pat)
    end

    instance_methods_to_class_methods(*instance_methods(false))
  end # Dir
end # VFS
